package provide objLab 0.4


#Pacote para controle de instrumento
#e bomba peristáltica baseado na filosofia
#de orientação à objeto


#Módulo objLab.tcl versão 04
#Incluído classes para controle de bomba
#peristáltica
#Percebemos a necessidade de incluir o uso de namespace
#para evitar conflitos com nomes de variáveis já existentes
#no programa bomba-0.3. Por exemplo o array status_bomba
#estava conflitando com a variável status_b do programa bomba
#Mas ao testar o uso do namespace percebemos que existe um 
#limite para variáveis e procedimentos dentro de um namespace
#Criamos então o namespace equip_lab apenas para as classes
#dedicadas à manipulação de equipamentos (bombas)
#No entanto o array status_bomba continuou conflitando 
#com a variável status_b pois ambos estão com escopo global 
#por isso trocamos status_bomba por bomba_status


#classe instrumento
#objeto eletrodo_pH
#objeto eletrodo_ORP
#objeto eletrodo_OD

#*******************************************************************
#Cada novo atributo é armazenado em um array para o qual devem
#ser criados os respectivos métodos para atualizar (def_...)
#e resgatar o valor atual (get_...)
#Os novos métodos devem ser inseridos na rotina cria_obj
#E na rotima remover_instrumento deve ser inserida os atributos
#para serem liberados da memória
#*******************************************************************


#*******************************************************************
#Atributos dos objetos da classe instrumento
#unidade é um array global que vai guardar o valor do atributo
#unidade do parâmetro e será indexado pelo nome do objeto
#e armazenando como valor a unidade de concentração 
#*******************************************************************

global unidade



#*******************************************************************
#estado_instrumento é um array global que vai guardar o valor do atributo
#estado do instrumento e será indexado pelo nome do objeto
#e poderá armazenar como valor uma das opções: monitorando, 
#parado, pausado, calibrando, condicionando
#*******************************************************************

global estado_instrumento



#*******************************************************************
#Leitura é um array global que vai guardar o valor do atributo
#última_leitura do instrumento e será indexado pelo nome do objeto
#armazenando como valor uma lista com o formato {tempo leitura}
#*******************************************************************

global leitura



#*******************************************************************
#procedimento para retornar a unidade usada pelo parâmetro associado
#ao respectivo sensor
#*******************************************************************

proc get_unidade { nome_sensor } {
    
    global unidade
    
    if { [info exists unidade($nome_sensor)] } {
	
	return $unidade($nome_sensor)
	
    } else {
	
	puts "Atenção: $nome_sensor não tem unidade"
	return "unidade_indefinida"
	
    }
    
}


#*******************************************************************
#Procedimento para definir a unidade do parâmetro associado ao sensor
#*******************************************************************

proc def_unidade { nome_sensor valor_unidade } {

global unidade

set unidade($nome_sensor) $valor_unidade

}


#*******************************************************************
#procedimento para retornar o estado atual do instrumento 
#*******************************************************************

proc get_estado_instrumento { nome_sensor } {
    
    global estado_instrumento
    
    if { [info exists estado_instrumento($nome_sensor)] } {
	    
	    return $estado_instrumento($nome_sensor)
	    
	} else {
	    
	    puts "Atenção: $nome_sensor não tem estado definido"
	    return "estado_instrumento_indefinido"
	    
	}
	
}

#*******************************************************************
#Procedimento para definir o estado atual do sensor
#*******************************************************************

proc def_estado_instrumento { nome_sensor valor_estado_instrumento } {

global estado_instrumento

set estado_instrumento($nome_sensor) $valor_estado_instrumento

}



#*******************************************************************
#Procedimento para definir a ultima leitura do sensor
#*******************************************************************

proc def_leitura { nome_sensor valor_leitura } {

global leitura

set leitura($nome_sensor) $valor_leitura

}


#*******************************************************************
#procedimento para retornar a leitura atual do sensor 
#*******************************************************************

proc get_leitura { nome_sensor } {
    
    global leitura
    
    if { [info exists leitura($nome_sensor)] } {
	
	return $leitura($nome_sensor)
	
    } else {
	
	puts "Atenção: $nome_sensor não tem leitura"
	return "leitura_indefinida"
	
    }
    
}





#*******************************************************************
#Processo de criação de um objeto da classe instrumento
#O argumento "args" é um nome especial que armazena todos 
#os argumentos restantes em uma lista de tamanho variável
#*******************************************************************


proc cria_instrumento { nome_instrumento comando args } {
    if { $comando == "get_unidade" } {
	    return [get_unidade $nome_instrumento]
	} elseif { $comando == "def_unidade"} {
	    def_unidade $nome_instrumento [lindex $args 0]
	} elseif { $comando == "get_estado_instrumento" } {
	    return [get_estado_instrumento $nome_instrumento]
	} elseif { $comando == "def_estado_instrumento"} {
	    def_estado_instrumento $nome_instrumento [lindex $args 0]
	} elseif { $comando == "get_leitura" } {
	    return [get_leitura $nome_instrumento]
	} elseif { $comando == "def_leitura"} {
	    def_leitura $nome_instrumento [lindex $args 0]
	} else {
	    puts "Erro: $comando é um comando desconhecido"
	}
}

#*******************************************************************
#Procedimento que cria a classe instrumento
#*******************************************************************

proc instrumento {args} {
    foreach nome $args {
#	puts "Criando instrumento $nome"
	proc $nome {comando args} "return \[eval cria_instrumento $nome \$comando \$args\]"
    }
}


proc remover_instrumento { args } {
    
    global unidade
    global estado_instrumento
    global leitura
    
    foreach nome $args {
	if { [info exists unidade($nome)] } {
	    unset unidade($nome)  ;#Deleta os dados do objeto
	}
	if { [info exists estado_instrumento($nome)] } {
	    unset estado_instrumento($nome)   ;#Deleta os dados do objeto
	}
	if { [info exists leitura($nome)] } {
	    unset leitura($nome) ;#Deleta os dados do objeto
	}
	rename $nome {}       ;#Deleta o comando objeto
    }
}





#***************************************************************************************
#A partir daqui, vamos criar os procedimentos da classe amostra
#a qual vai possuir os seguintes atributos/propriedades:
#projeto {codigo coordenador}
#experimento {numero estado} estado (iniciando andamento pausado interrompido realizado)
#coleta {{local} data}
#responsavel_experimento
#parametros_monitorados {pH lim_inf lim_sup ORP lim_inf lim_sup etc}
#**************************************************************************************


#*******************************************************************
#Atributos dos objetos da classe amostra
#durante um projeto serão realizados vários experimentos
#e cada experimento irá gerar várias amostras
#que serão criadas como objetos e descartadas ao final de 
#cada análise, cujo nome será o próprio código da amostra
#Ou seja, o nome de cada objeto "amostra" será o próprio código da amostra
#As variáveis a seguir serão arrays globais que vão guardar os 
#atributos das amostras e será indexado pelo codigo de cada amostra
#Será usada a hora do sistema no formato original como código de
#identificação de cada amostra
#Quanto ao responsável pelo experimento seria mais conveniente usar a chave para
#padronizar o armazenamento e a busca
#*******************************************************************

#projeto {codigo coordenador}
#experimento {numero estado_atual (iniciando andamento pausado interrompido realizado)}
#coleta {local data responsavel}
#responsavel_experimento
#parametros_monitorados {p1 lim_inf lim_sup p2 lim_inf lim_sup p3 lim_inf lim_sup ...}

#O array data_coleta é desnessário pois o codigo da amostra já
#armazena as informações de dia e hora da coleta
global projeto
global experimento 
global coleta
global responsavel_experimento
global parametros_monitorados


#*******************************************************************
#Deixamos explícito os dois argumentos para que o retorno 
#das informações do projeto fossem no formato de lista
#*******************************************************************

proc def_projeto { cod_amostra codigo coordenador } {
    
    global projeto
    
    set projeto($cod_amostra) [list $codigo $coordenador]
    
}


proc get_projeto { cod_amostra } {
    
    global projeto
    
    if { [info exists projeto($cod_amostra)] } {
	    
	    return $projeto($cod_amostra)
	    
	} else {
	    
	    puts "Atenção:$cod_amostra não tem projeto associado"
	    return "sem_projeto_associado"
	}
}


#*******************************************************************
#Deixamos explícito os dois argumentos para que o retorno 
#das informações do experimento fossem no formato de lista
#*******************************************************************


proc def_experimento { cod_amostra codigo estado } {
    
    global experimento
    
    set experimento($cod_amostra) [list $codigo $estado]
    
}

proc get_experimento { cod_amostra } {
    
    global experimento
    
    if { [info exists experimento($cod_amostra)] } {
	    return $experimento($cod_amostra)
	    
	} else {
	    
	    puts "Atenção: $cod_amostra não tem experimento associado"
	    return "sem_experimento_associado"
	}
}

#*******************************************************************
#Neste caso foi incluido apenas o local de amostragem pois o código
#da amostra já contém as informações de data
#*******************************************************************


proc def_coleta { cod_amostra local data responsavel} {
    
    global coleta
    
    set coleta($cod_amostra) [list $local $data $responsavel]
    
}

proc get_coleta { cod_amostra } {
    
    global coleta
    
    if { [info exists coleta($cod_amostra)] } {
	    return $coleta($cod_amostra)
	    
	} else {
	    
	    puts "Atenção: $cod_amostra não tem local  e data associado"
	    return "local_coleta_indefinido"
	}
}


proc def_responsavel_experimento { cod_amostra resp_exp } {
    
    global responsavel_experimento 
    
    set responsavel_experimento($cod_amostra) $resp_exp
    
}


proc get_responsavel_experimento { cod_amostra } {
    
    global responsavel_experimento 
    
    if { [info exists responsavel_experimento($cod_amostra)] } {
	   
	    return $responsavel_experimento($cod_amostra)
	    
	} else {
	    
	    puts "Atenção:$cod_amostra não tem um responsavel associado"
	    return "sem_responsavel"
	}
}


#*******************************************************************
#Neste caso o número de parâmetros é variável 
#por isso deixamos aberto usando o argumento "args"
#Incluimos o comando lindex 2 vezes para incluir
#apenas uma lista de N elementos dispensando
#posterior tratamento dos dados retornados pela função get_
#*******************************************************************


proc def_parametros_monitorados { cod_amostra args } {
    
    global parametros_monitorados 
    

    set parametros_monitorados($cod_amostra) [lindex [lindex $args 0] 0]
}

proc get_parametros_monitorados { cod_amostra } {
    
    global parametros_monitorados 
    
    if { [info exists parametros_monitorados($cod_amostra)] } {
	   
	return $parametros_monitorados($cod_amostra)
	    
	} else {
	    
	    puts "Atenção:$cod_amostra não tem parâmetro associado"
	    return "sem_parâmetros"
	}
}


#*******************************************************************
#Processo de criação de um objeto da classe amostra
#O argumento "args" é um nome 
#especial que armazena todos os argumentos restantes em uma lista
#*******************************************************************


proc cria_amostra { cod_amostra comando args } {

    if { $comando == "get_projeto" } {
	    return [get_projeto $cod_amostra]
	} elseif { $comando == "def_projeto"} {
	    def_projeto $cod_amostra [lindex $args 0] [lindex $args 1]

	} elseif { $comando == "get_experimento" } {
	    return [get_experimento $cod_amostra]
	} elseif { $comando == "def_experimento"} {
	    def_experimento $cod_amostra [lindex $args 0] [lindex $args 1]

	} elseif { $comando == "get_coleta" } {
	    return [get_coleta $cod_amostra]
	} elseif { $comando == "def_coleta"} {
	    def_coleta $cod_amostra [lindex $args 0] [lindex $args 1] [lindex $args 2]

	} elseif { $comando == "get_responsavel_experimento" } {
	    return [get_responsavel_experimento $cod_amostra]
	} elseif { $comando == "def_responsavel_experimento"} {
	    def_responsavel_experimento $cod_amostra [lindex $args 0]

	} elseif { $comando == "get_parametros_monitorados" } {
	    return [get_parametros_monitorados $cod_amostra]
	} elseif { $comando == "def_parametros_monitorados"} {
	    def_parametros_monitorados $cod_amostra $args

	} else {
	    puts "Erro: $comando é um comando desconhecido"
	}
}

#*******************************************************************
#Procedimento que cria a classe amostra
#*******************************************************************

proc amostra {args} {
    foreach codigo $args {
	proc $codigo {comando args} "return \[eval cria_amostra $codigo \$comando \$args\]"
    }
}


proc remover_amostra { args } {
    
    global projeto
    global experimento 
    global coleta
    global responsavel_experimento
    global parametros_monitorados

    
    foreach nome $args {
	if { [info exists projeto($nome)] } {
	    unset projeto($nome)  ;#Deleta os dados do objeto
	}
	if { [info exists experimento($nome)] } {
	    unset experimento($nome)   ;#Deleta os dados do objeto
	}
	if { [info exists coleta($nome)] } {
	    unset coleta($nome) ;#Deleta os dados do objeto
	}
	if { [info exists responsavel_experimento($nome)] } {
	    unset responsavel_experimento($nome) ;#Deleta os dados do objeto
	}
	if { [info exists parametros_monitorados($nome)] } {
	    unset parametros_monitorados($nome) ;#Deleta os dados do objeto
		    }
	rename $nome {}       ;#Deleta o comando objeto
    }
}



#*******************************************************************
#Procedimento de criação da classe medida
#Cada objeto da classe "medida" (measurement) vai representar 
#um resultado analítico compreendendo uma ou várias leituras, 
#com os respectivos tempos, de um determinado parâmetro vinculado 
#a uma amostra através do código da amostra
#O nome da cada instância de medida pode ser o nome do parâmetro
#Identifiquei a necessidade desta classe pois para cada amostra 
#serão geradas mais de uma medida simultâmeamente
#Essa classe poderá acomodar futuramente outros atributos tais
#como curva de calibração
#*******************************************************************

#O nome do objeto já é o nome do parâmetro monitorado
#amostra_medida { cod_amostra } ;#código da amostra
#parametro_medido ( parametro }
#leituras { tempo leitura } ;#leitura


#Arrays globais que vão armazenar os respectivos atributos
#das medidas
global amostra_medida
global parametro_medido
global parametro_unidade
global leituras



proc def_amostra_medida { cod_medida amostra } {
    
    global amostra_medida
    
    set amostra_medida($cod_medida) $amostra
    
}


proc get_amostra_medida { cod_medida } {
    
    global amostra_medida
    
    if { [info exists amostra_medida($cod_medida)] } {
	
	return $amostra_medida($cod_medida)
	
    } else {
	
	puts "Atenção $cod_medida não tem uma amostra associada"
	return "sem_amostra"
    }
}


proc def_parametro_medido { cod_medida parametro } {
    
    global parametro_medido
    
    set parametro_medido($cod_medida) $parametro
    
}



proc get_parametro_medido { cod_medida } {
    
    global parametro_medido
    
    if { [info exists parametro_medido($cod_medida)] } {
	
	return $parametro_medido($cod_medida)
	
    } else {
	
	puts "Atenção $cod_medida não tem parâmetro medido"
	
    }
}


proc def_parametro_unidade { cod_medida unidade } {
    
    global parametro_unidade
    
    set parametro_unidade($cod_medida) $unidade
    
}



proc get_parametro_unidade { cod_medida } {
    
    global parametro_unidade
    
    if { [info exists parametro_unidade($cod_medida)] } {
	
	return $parametro_unidade($cod_medida)
	
    } else {
	
	puts "Atenção $cod_medida não tem unidade associada"
	
    }
}


proc def_leituras { cod_medida args } {
    
    global leituras   

    set leituras($cod_medida) [lindex $args 0]
    
}

proc get_leituras { cod_medida } {
    
    global leituras
    
    if { [info exists leituras($cod_medida)] } {
	return $leituras($cod_medida)
	
    } else {
	
	puts "Atenção $cod_medida não tem leitura"
	
    }
    
}

#*******************************************************************
#Processo de criação de um objeto da classe "medida"
#*******************************************************************

proc cria_medida { cod_medida comando args } {

    if { $comando == "get_amostra_medida" } {
	    return [get_amostra_medida $cod_medida]

	} elseif { $comando == "def_amostra_medida"} {
	    def_amostra_medida $cod_medida [lindex $args 0]

	} elseif { $comando == "get_parametro_medido" } {
	    return [get_parametro_medido $cod_medida]

	} elseif { $comando == "def_parametro_medido"} {
	    def_parametro_medido $cod_medida [lindex $args 0]

	} elseif { $comando == "get_parametro_unidade" } {
	    return [get_parametro_unidade $cod_medida]

	} elseif { $comando == "def_parametro_unidade"} {
	    def_parametro_unidade $cod_medida [lindex $args 0]

	} elseif { $comando == "get_leituras" } {
	    return [get_leituras $cod_medida]

	} elseif { $comando == "def_leituras"} {
	    def_leituras $cod_medida $args

	} else {
	    puts "Erro: $comando é um comando desconhecido"
	}
}


#*******************************************************************
#Procedimento que cria a classe medida
#*******************************************************************

proc medida {args} {
    foreach codigo $args {
	proc $codigo {comando args} "return \[eval cria_medida $codigo \$comando \$args\]"
    }
}


#*******************************************************************
#Procedimento para remover os objetos da classe medida
#*******************************************************************


proc remover_medida { args } {
    
    global amostra_medida
    global parametro_medido
    global parametro_unidade
    global leituras
    
    
    foreach nome $args {
	if { [info exists amostra_medida($nome)] } {
	    unset amostra_medida($nome)  ;#Deleta os dados do objeto
	}
	if { [info exists parametro_medido($nome)] } {
	    unset parametro_medido($nome)   ;#Deleta os dados do objeto
	}
	if { [info exists parametro_unidade($nome)] } {
	    unset parametro_unidade($nome)   ;#Deleta os dados do objeto
	}	
	if { [info exists leituras($nome)] } {
	    unset leituras($nome) ;#Deleta os dados do objeto
	}
	rename $nome {}       ;#Deleta o comando objeto
    }
}

##########################################################################
#Criação do namespace equip_lab para organizar os nomes das classes
#referentes aos equipamentos de laboratório
##########################################################################

namespace eval equip_lab {


#*******************************************************************
#Definição das classes para controle de bombas e operações de bombeio
#Classe bomba_peristaltica
#objeto ismatec
#objeto masterflex
#objeto watson_marlow
#...
#*******************************************************************


#*******************************************************************
#Cada novo atributo é armazenado em um array para o qual devem
#ser criados os respectivos métodos para atualizar (def_...)
#e resgatar o valor atual (get_...)
#Os novos métodos devem ser inseridos na rotina criar_bomba_peristaltica
#E na rotima remover_bomba_peritaltica deve ser inserida os atributos
#para serem liberados da memória
#*******************************************************************


#*******************************************************************
#Atributos dos objetos da classe bomba_peristaltica
#ID_bomba_per é um identificador único que permite controlar mais de uma 
#bomba pela mesma porta serial
#ID_bomba_per é um array global que vai guardar o valor do atributo 
#ID_bomba_per e será indexado pelo nome do objeto e armazenando como
#valor um identificador único
#*******************************************************************

global ID_bomba_per

#*******************************************************************
#Procedimento para retornar a ID_bomba_per para a qual será usado
#um número sequencial
#*******************************************************************

proc get_ID_bomba_per { nome_bomba } {
    
    global ID_bomba_per
    
    if { [info exists ID_bomba_per($nome_bomba)] } {
	
	return $ID_bomba_per($nome_bomba)
	
    } else {
	
	puts "Atenção: $nome_bomba não tem numeracao"
	return "bomba_sem_numeracao"
	
    }
    
}

#*******************************************************************
#Procedimento para definir o numero da bomba
#*******************************************************************



proc def_ID_bomba_per { nome_bomba num_ID } {

global ID_bomba_per

set ID_bomba_per($nome_bomba) $num_ID

}



#*******************************************************************
#ID_marca_modelo_bomba_per { marca modelo }– marca e modelo da bomba 
#peristáltica
#*******************************************************************

global marca_modelo_bomba_per 


#*******************************************************************
#Procedimento para definir a marca e o modelo da bomba
#*******************************************************************

proc def_marca_modelo_bomba_per { nome_bomba marca_modelo } {

global marca_modelo_bomba_per

set marca_modelo_bomba_per($nome_bomba) $marca_modelo

}

#*******************************************************************
#Procedimento para retornar a marca e o modelo da bomba
#*******************************************************************

proc get_marca_modelo_bomba_per { nome_bomba } {
    
    global marca_modelo_bomba_per
    
    if { [info exists marca_modelo_bomba_per($nome_bomba)] } {
	
	return $marca_modelo_bomba_per($nome_bomba)
	
    } else {
	
	puts "Atenção: $nome_bomba não tem marca e modelo definidos"
	return "sem_marca_modelo"
	
    }
    
}




#*******************************************************************
#num_canais_bomba { numero } – número de canais usados na bomba
#peristáltica
#*******************************************************************

global num_canais_bomba

#*******************************************************************
#Procedimento para definir o número de canais da bomba
#*******************************************************************

proc def_num_canais_bomba { nome_bomba num_canais } {

global num_canais_bomba

set num_canais_bomba($nome_bomba) $num_canais

}


#*******************************************************************
#Procedimento para retornar o número de canais da bomba
#*******************************************************************

proc get_num_canais_bomba { nome_bomba } {
    
    global num_canais_bomba
    
    if { [info exists num_canais_bomba($nome_bomba)] } {
	
	return $num_canais_bomba($nome_bomba)
	
    } else {
	
	puts "Atenção: $nome_bomba não possui o numero de canais definido"
	return "sem_num_canais"
	
    }
    
}



#*******************************************************************
#porta_serial_bomba {nome} nome da porta serial utilizada pela bomba
#*******************************************************************

global porta_serial_bomba


#*******************************************************************
#Procedimento para definir a porta serial usada pela bomba
#*******************************************************************

proc def_porta_serial_bomba { nome_bomba porta_serial } {

global porta_serial_bomba

set porta_serial_bomba($nome_bomba) $porta_serial

}


#*******************************************************************
#Procedimento para retornar o descritor da porta serial usada pela 
#bomba bomba
#*******************************************************************

proc get_porta_serial_bomba { nome_bomba } {
    
    global porta_serial_bomba
    
    if { [info exists porta_serial_bomba($nome_bomba)] } {
	
	return $porta_serial_bomba($nome_bomba)
	
    } else {
	
	puts "Atenção: $nome_bomba ainda não possui \
uma porta serial associada."
	return "sem_porta_serial"
	
    }
    
}



#*******************************************************************
#bomba_status – {LIGADO, INICIANDO, OCIOSO, CONFIGURANDO, OPERACIONAL,
#EXECUTANTO, LIBERANDO, PAUSANDO, PAUSADO, PARADO} 
#diferentes estados do bomba conforme recomendações da norma Lecis.
#*******************************************************************

global bomba_status

#*******************************************************************
#Procedimento para definir o número de canais da bomba
#*******************************************************************

proc def_bomba_status { nome_bomba status } {

global bomba_status

set bomba_status($nome_bomba) $status

}

#*******************************************************************
#Procedimento para retornar o número de canais da bomba
#*******************************************************************

proc get_bomba_status { nome_bomba } {
    
    global bomba_status
    
    if { [info exists bomba_status($nome_bomba)] } {
	
	return $bomba_status($nome_bomba)
	
    } else {
	
	puts "Atenção: $nome_bomba não possui estado definido"
	return "sem_bomba_status"
	
    }
    
}

#*******************************************************************
#sentido_rotacao_bomba - { horario anti_horario }
#*******************************************************************

global sentido_rotacao_bomba


#*******************************************************************
#Procedimento para definir o sentido de rotação da bomba
#podendo ser arbitrariamente horario ou anti_horario
#*******************************************************************

proc def_sentido_rotacao_bomba { nome_bomba sentido } {

global sentido_rotacao_bomba

set sentido_rotacao_bomba($nome_bomba) $sentido

}

#*******************************************************************
#Procedimento para retornar o sentido de rotacao da bomba
#*******************************************************************

proc get_sentido_rotacao_bomba { nome_bomba } {
    
    global sentido_rotacao_bomba
    
    if { [info exists sentido_rotacao_bomba($nome_bomba)] } {
	
	return $sentido_rotacao_bomba($nome_bomba)
	
    } else {
	
	puts "Atenção: $nome_bomba não possui sentido de rotacao definido"
	return "sem_sentido_rotacao_bomba"
	
    }
    
}




#*******************************************************************
#tubo_modelo - { cte_tubo } constante para ser usada na conversao
#da vazão em velocidade de rotacao pela classe formula de conversao
#*******************************************************************


global tubo_modelo


#*******************************************************************
#Procedimento para definir o sentido de rotação da bomba
#podendo ser arbitrariamente horario ou anti_horario
#*******************************************************************

proc def_tubo_modelo { nome_bomba tubo } {

global tubo_modelo

set tubo_modelo($nome_bomba) $tubo

}

#*******************************************************************
#Procedimento para retornar o sentido de rotacao da bomba
#*******************************************************************

proc get_tubo_modelo { nome_bomba } {
    
    global tubo_modelo
    
    if { [info exists tubo_modelo($nome_bomba)] } {
	
	return $tubo_modelo($nome_bomba)
	
    } else {
	
	puts "Atenção: $nome_bomba não possui modelo de tubo definido"
	return "sem_tubo_modelo"
	
    }
    
}

#*******************************************************************
#Procedimento iniciar_bomba
#Este comando se refere apenas à classe bomba e exige apenas
#que a mesma esteja no estado "OPERACIONAL" e que exista
#um identificador para a porta serial aberta e configurada
#pela instância da classe porta_serial
#*******************************************************************

proc iniciar_bomba { nome_bomba comando_iniciar } {
    
    global porta_serial_bomba
    global bomba_status
    
    
    set teste_id_serial \
	[catch {set id_serial [get_ID_porta_serial $porta_serial_bomba($nome_bomba)]}]
    
    if { !($teste_id_serial) } {
	
	if { (![catch { tell $id_serial }]) && \
		 ($bomba_status($nome_bomba) == "OPERACIONAL")} {
	    
	    puts "Existe Porta serial $id_serial aberta e o status de $nome_bomba é OPERACIONAL"
	    puts "Enviando comando iniciar $comando_iniciar"
	    puts $id_serial $comando_iniciar
	    flush $id_serial
	    puts "Configurando status da bomba \"EXECUTANDO\""
	    set bomba_status($nome_bomba) "EXECUTANDO"
	    return "comando_iniciar_aceito"
	} else {
	    puts "Atenção: status de  $nome_bomba não é \"OPERACIONAL\"."
	    return "comando_iniciar_rejeitado"	
	    
	}
    } else {
	puts "Atenção: $nome_bomba ainda não possui \
uma porta serial associada."
	return "comando_iniciar_rejeitado"
    }
}


#*******************************************************************
#Procedimento parar_bomba
#Este comando se refere apenas à classe bomba e exige apenas
#que a mesma esteja no estado "EXECUTANDO" e que exista
#um identificador para a porta serial aberta e configurada
#pela instância da classe porta_serial
#*******************************************************************

proc parar_bomba { nome_bomba comando_parar } {
    
    global porta_serial_bomba
    global bomba_status
   
set teste_id_serial \
	[catch {set id_serial [get_ID_porta_serial $porta_serial_bomba($nome_bomba)]}]
 
    if { !($teste_id_serial) } {
	
	if { (![catch {tell $id_serial}]) && \
		 ($bomba_status($nome_bomba) == "EXECUTANDO")} {
	    
	    puts "Existe Porta serial $id_serial aberta e o status é EXECUTANDO"
	    puts "Enviando comando parar $comando_parar"
	    puts $id_serial $comando_parar
	    flush $id_serial
	    puts "Configurando status da bomba \"OPERACIONAL\""
	    set bomba_status($nome_bomba) "OPERACIONAL"
	    return "comando_parar_aceito"
	} else {
	    puts "Atenção: $nome_bomba ainda não possui \
uma porta serial associada e/ou status não é EXECUTANDO."
	    return "comando_parar_rejeitado"	
	    
	}

    } else {
	puts "Atenção: $nome_bomba ainda não possui \
uma porta serial associada."
	return "comando_parar_rejeitado"
    }    
    
}

#*******************************************************************
#Processo de criação de um objeto da classe bomba_peristaltica
#O argumento "args" é um nome especial que armazena todos 
#os argumentos restantes em uma lista de tamanho variável
#*******************************************************************

proc criar_bomba_peristaltica { nome_bomba comando args } {
    if { $comando == "get_ID_bomba_per" } {
	    return [get_ID_bomba_per $nome_bomba]
	} elseif { $comando == "def_ID_bomba_per"} {
	    def_ID_bomba_per $nome_bomba [lindex $args 0]
	} elseif { $comando == "get_marca_modelo_bomba_per" } {
	    return [get_marca_modelo_bomba_per $nome_bomba]
	} elseif { $comando == "def_marca_modelo_bomba_per"} {
	    def_marca_modelo_bomba_per $nome_bomba [lindex $args 0]
	} elseif { $comando == "get_num_canais_bomba" } {
	    return [get_num_canais_bomba $nome_bomba]
	} elseif { $comando == "def_num_canais_bomba"} {
	    def_num_canais_bomba $nome_bomba [lindex $args 0]
	} elseif { $comando == "get_porta_serial_bomba" } {
	    return [get_porta_serial_bomba $nome_bomba]
	} elseif { $comando == "def_porta_serial_bomba"} {
	    def_porta_serial_bomba $nome_bomba [lindex $args 0]
	} elseif { $comando == "get_bomba_status" } {
	    return [get_bomba_status $nome_bomba]
	} elseif { $comando == "def_bomba_status"} {
	    def_bomba_status $nome_bomba [lindex $args 0]
	} elseif { $comando == "get_sentido_rotacao_bomba" } {
	    return [get_sentido_rotacao_bomba $nome_bomba]
	} elseif { $comando == "def_sentido_rotacao_bomba"} {
	    def_sentido_rotacao_bomba $nome_bomba [lindex $args 0]
	} elseif { $comando == "get_tubo_modelo" } {
	    return [get_tubo_modelo $nome_bomba]
	} elseif { $comando == "def_tubo_modelo"} {
	    def_tubo_modelo $nome_bomba [lindex $args 0]
	} elseif { $comando == "iniciar_bomba"} {
	    iniciar_bomba $nome_bomba [lindex $args 0]
	} elseif { $comando == "parar_bomba"} {
	    parar_bomba $nome_bomba [lindex $args 0]
	} else {
	    puts "Erro: $comando é um comando desconhecido"
	}
}



#*******************************************************************
#Procedimento que cria a classe bomba_peristaltica
#*******************************************************************

proc bomba_peristaltica {args} {
    foreach nome $args {
	puts "Criando bomba $nome"
	proc $nome {comando args} "return \[eval criar_bomba_peristaltica $nome \$comando \$args\]"
    }
}

proc remover_bomba_peristaltica { args } {
    
    global ID_bomba_per
    global marca_modelo_bomba_per 
    global num_canais_bomba
    global porta_serial_bomba
    global bomba_status
    global sentido_rotacao_bomba
    global tubo_modelo
    
    
    foreach nome $args {
	if { [info exists ID_bomba_per($nome)] } {
	    unset ID_bomba_per($nome)  ;#Deleta os dados do objeto
	}
	if { [info exists marca_modelo_bomba_per($nome)] } {
	    unset marca_modelo_bomba_per($nome)   ;#Deleta os dados do objeto
	}
	if { [info exists num_canais_bomba($nome)] } {
	    unset num_canais_bomba($nome) ;#Deleta os dados do objeto
	}
	if { [info exists porta_serial_bomba($nome)] } {
	    unset porta_serial_bomba($nome) ;#Deleta os dados do objeto
	}
	if { [info exists bomba_status($nome)] } {
	    unset bomba_status($nome) ;#Deleta os dados do objeto
	}
	if { [info exists sentido_rotacao_bomba($nome)] } {
	    unset sentido_rotacao_bomba($nome) ;#Deleta os dados do objeto
	}
	if { [info exists tubo_modelo($nome)] } {
	    unset tubo_modelo($nome) ;#Deleta os dados do objeto
	}
	rename $nome {}       ;#Deleta o comando objeto

	puts "Removendo $nome"
    }
}

}

#********************************************************************
#Definição da classe porta_serial para padronizar a abertura e 
#configuração de portas seriais a serem usadas tanto por instrumentos
#como para equipamentos
#********************************************************************

#*******************************************************************
#Atributos dos objetos da classe porta_serial
#ID_porta_serial é um identificador único que permite abrir mais
#de uma porta serial
#nome_disp_serial /dev/ttyS*  ou com:
#numero_disp_serial
#par_com_porta_serial - { baud, paridade, bit_dados, bit_parada }
#status_porta_serial {
#SERIAL_OPEN               Porta serial aberta
#SERIAL_ERRO_OPEN        Nao foi possivel abrir a porta serial.
#SERIAL_ERRO_CLOSE       Nao foi possivel fechar a porta serial.
#SERIAL_ERRO_READ        Nao foi possivel ler a porta serial.
#SERIAL_ERRO_WRITE       Nao foi possivel escrever na porta serial.
#SERIAL_ERRO_TIMEOUT     Ocorreu um Time Out ao tentar ler a porta serial }
#*******************************************************************

global ID_porta_serial

#*******************************************************************
#Procedimento para retornar a ID_porta_serial para a qual será usado
#identificador da porta serial (file1, file2 etc)
#*******************************************************************

proc get_ID_porta_serial { nome_obj_serial } {
    
    global ID_porta_serial
    
    if { [info exists ID_porta_serial($nome_obj_serial)] } {
	
	return $ID_porta_serial($nome_obj_serial)
	
    } else {
	
	puts "Atenção: $nome_obj_serial não tem numeracao"
	return "porta_sem_numeracao"
	
    }
    
}

#*******************************************************************
#Procedimento para definir o Identificador da porta serial
#*******************************************************************


proc def_ID_porta_serial { nome_obj_serial ID } {

global ID_porta_serial

set ID_porta_serial($nome_obj_serial) $ID

}


global nome_disp_serial

#*******************************************************************
#Procedimento para retornar o nome do dispositivo físico serial
#*******************************************************************

proc get_nome_disp_serial { nome_obj_serial } {
    
    global nome_disp_serial
    
    if { [info exists nome_disp_serial($nome_obj_serial)] } {
	
	return $nome_disp_serial($nome_obj_serial)
	
    } else {
	
	puts "Atenção: $nome_obj_serial não tem um dispositivo definido"
	return "porta_sem_nome_disp_serial"
	
    }
    
}

#*******************************************************************
#Procedimento para definir o nome do dispositivo serial
#*******************************************************************


proc def_nome_disp_serial { nome_obj_serial disp } {

global nome_disp_serial

set nome_disp_serial($nome_obj_serial) $disp

}


global numero_disp_serial

#*******************************************************************
#Procedimento para retornar o numero do dispositivo físico serial
#*******************************************************************

proc get_numero_disp_serial { nome_obj_serial } {
    
    global numero_disp_serial
    
    if { [info exists numero_disp_serial($nome_obj_serial)] } {
	
	return $numero_disp_serial($nome_obj_serial)
	
    } else {
	
	puts "Atenção: $nome_obj_serial não tem número para o dispositivo"
	return "porta_sem_numero_disp_serial"
	
    }
    
}

#*******************************************************************
#Procedimento para definir o numero do dispositivo serial
#*******************************************************************


proc def_numero_disp_serial { nome_obj_serial num } {

global numero_disp_serial

set numero_disp_serial($nome_obj_serial) $num

}

global par_com_porta_serial

#*******************************************************************
#Procedimento para retornar os parâmetros de comunicação da porta 
#*******************************************************************

proc get_par_com_porta_serial { nome_obj_serial } {
    
    global par_com_porta_serial
    
    if { [info exists par_com_porta_serial($nome_obj_serial)] } {
	
	return $par_com_porta_serial($nome_obj_serial)
	
    } else {
	
	puts "Atenção: $nome_obj_serial não tem par_com definidos"
	return "porta_sem_par_com"
	
    }
    
}

#*******************************************************************
#Procedimento para definir os parâmetros de comunicação da 
#porta serial. Os parametros devem entrar no forma de uma lista
#*******************************************************************


proc def_par_com_porta_serial { nome_obj_serial par_com } {

global par_com_porta_serial

set par_com_porta_serial($nome_obj_serial) $par_com

}

global status_porta_serial


#*******************************************************************
#Procedimento para retornar o status da porta serial
#status_porta_serial {
#SERIAL_OPEN             Porta serial aberta
#SERIAL_CLOSED           Porta serial fechada
#SERIAL_ERRO_OPEN        Nao foi possivel abrir a porta serial.
#SERIAL_ERRO_CLOSE       Nao foi possivel fechar a porta serial.
#SERIAL_ERRO_READ        Nao foi possivel ler a porta serial.
#SERIAL_ERRO_WRITE       Nao foi possivel escrever na porta serial.
#SERIAL_ERRO_TIMEOUT     Ocorreu um Time Out ao tentar ler a porta serial 
#SERIAL_ERRO_CONFIG      Não foi possível configurar os par_com da porta }
#*******************************************************************

proc get_status_porta_serial { nome_obj_serial } {
    
    global status_porta_serial
    
    if { [info exists status_porta_serial($nome_obj_serial)] } {
	
	return $status_porta_serial($nome_obj_serial)
	
    } else {
	
	puts "Atenção: $nome_obj_serial não tem status definidos"
	return "porta_sem_status"
	
    }
    
}

#*******************************************************************
#Procedimento para definir o status da porta serial
#*******************************************************************


proc def_status_porta_serial { nome_obj_serial status } {

global status_porta_serial

set status_porta_serial($nome_obj_serial) $status

}

#*******************************************************************
#Procedimento para criar um identificador para uma porta serial
#Este procedimento verifica se já existe um identificador e fecha
#a porta caso esteja aberta.
#Em seguida verifica o conteúdo dos atributos nome, número e par_com
#para abrir a porta, configurar e definir o estado da porta
#*******************************************************************


proc porta_serial_abrir { nome_obj_serial } {
    
    global ID_porta_serial 
    global nome_disp_serial 
    global numero_disp_serial 
    global par_com_porta_serial 
    global status_porta_serial 
    
    
    
    if { [info exists ID_porta_serial($nome_obj_serial)] } {
	puts "Existe ID da porta serial"
	#O comando catch retorna 0 para canal aberto e 1 para canal fechado
	#
	set id_serial ID_porta_serial($nome_obj_serial)
	
	if {![catch {tell $id_serial}]} {
	    puts "$id_serial está aberta!"
	    if {[catch {close $id_serial}]} {
		set status_porta_serial($nome_obj_serial) "SERIAL_ERRO_CLOSE"
	    } else {
		set status_porta_serial($nome_obj_serial) "SERIAL_CLOSED"
		puts "id_serial está fechada"
	    }   
	    
	    
	    
	    puts "$id_serial está fechada!"
	    set ID_porta_serial($nome_obj_serial) ""
	    
	}
	
    }
    
    if { [info exists nome_disp_serial($nome_obj_serial)] } {
	
	if { [info exists numero_disp_serial($nome_obj_serial)] } {
	    
	    if {[info exists par_com_porta_serial($nome_obj_serial)]} {
		
		set nome [append nome $nome_disp_serial($nome_obj_serial) \
			      $numero_disp_serial($nome_obj_serial)]
		
		set ret_abrir [ catch { set ID_porta_serial($nome_obj_serial)\
					    [open $nome r+] } ]
		
		set ret_conf [catch {fconfigure $ID_porta_serial($nome_obj_serial)\
					 -mode $par_com_porta_serial($nome_obj_serial) }]
		
		puts  "Nome -> $nome_disp_serial($nome_obj_serial)"
		puts "Numero -> $numero_disp_serial($nome_obj_serial)"
		puts "Nome da porta serial -> $nome"
		#	puts "ID da porta serial $ID_porta_serial($nome_obj_serial)"
		puts "par_com -> $par_com_porta_serial($nome_obj_serial)"
		puts "abrir -> $ret_abrir e configurar  -> $ret_conf"
		
		if {$ret_abrir} {
		    
		    set status_porta_serial($nome_obj_serial) "SERIAL_ERRO_OPEN"
		    
		} else {
		    
		    if {$ret_conf} {
			set status_porta_serial($nome_obj_serial) "SERIAL_ERRO_CONFIG"
		    } else {
			set status_porta_serial($nome_obj_serial) "SERIAL_OPEN"
		    }	
		}
		
	    } else {
		puts "Atenção: $nome_obj_serial não tem par_com definidos"
		return "porta_sem_par_com"
    	    }
	    
	} else {
	    puts "Atenção: $nome_obj_serial não tem número para o dispositivo"
	    return "porta_sem_numero_disp_serial"
	}
	
    } else {
	puts "Atenção: $nome_obj_serial não tem um dispositivo definido"
	return "porta_sem_nome_disp_serial"
    }
    
}

#*******************************************************************
#Procedimento para fechar uma porta serial
#*******************************************************************


proc porta_serial_fechar { nome_obj_serial } {
    
    global ID_porta_serial
    global status_porta_serial 
    
    if { [info exists ID_porta_serial($nome_obj_serial)] } {
	puts "Existe ID da porta serial"
	
	#O comando catch retorna 0 para canal aberto e 1 para canal fechado
	
	set id_serial $ID_porta_serial($nome_obj_serial)

	puts "id_serial -> $id_serial"

	if {![catch {tell $id_serial}]} {
	    puts "$id_serial está aberta!"
	    
	    if {[catch {close $id_serial}]} {
		set status_porta_serial($nome_obj_serial) "SERIAL_ERRO_CLOSE"
	    } else {
		set status_porta_serial($nome_obj_serial) "SERIAL_CLOSED"
		puts "id_serial está fechada"
		set ID_porta_serial($nome_obj_serial) ""
	    }
	}
	
    }
}

#*******************************************************************
#Processo de criação de um objeto da classe porta_serial
#*******************************************************************

proc criar_porta_serial { nome_porta comando args } {
    if { $comando == "get_ID_porta_serial" } {
	    return [get_ID_porta_serial $nome_porta]
	} elseif { $comando == "def_ID_porta_serial"} {
	    def_ID_porta_serial $nome_porta [lindex $args 0]
	} elseif { $comando == "get_nome_disp_serial" } {
	    return [get_nome_disp_serial $nome_porta]
	} elseif { $comando == "def_nome_disp_serial"} {
	    def_nome_disp_serial $nome_porta [lindex $args 0]
	} elseif { $comando == "get_numero_disp_serial" } {
	    return [get_numero_disp_serial $nome_porta]
	} elseif { $comando == "def_numero_disp_serial"} {
	    def_numero_disp_serial $nome_porta [lindex $args 0]
	} elseif { $comando == "get_par_com_porta_serial" } {
	    return [get_par_com_porta_serial $nome_porta]
	} elseif { $comando == "def_par_com_porta_serial"} {
	    def_par_com_porta_serial $nome_porta [lindex $args 0]
	} elseif { $comando == "get_status_porta_serial" } {
	    return [get_status_porta_serial $nome_porta]
	} elseif { $comando == "def_status_porta_serial"} {
	    def_status_porta_serial $nome_porta [lindex $args 0]
	} elseif { $comando == "porta_serial_abrir" } {
	    porta_serial_abrir $nome_porta
	} elseif { $comando == "porta_serial_fechar" } {
	    porta_serial_fechar $nome_porta
	} else {
	    puts "Erro: $comando é um comando desconhecido"
	}
}

#*******************************************************************
#Procedimento que cria a classe porta_serial
#*******************************************************************

proc porta_serial {args} {
    foreach nome $args {
	puts "Criando porta_serial $nome"
	proc $nome {comando args} "return \[eval criar_porta_serial $nome \$comando \$args\]"
    }
}

#*******************************************************************
#Incluimos o comando close $ID_porta_serial($nome) para fechar
#automaticamente o identificador da porta serial para liberar
#todos os recursos alocados pela porta_serial
#evitando o uso do comando porta_serial_fechar antes de remover
#objeto porta_serial
#*******************************************************************

proc remover_porta_serial { args } {
    
    global ID_porta_serial 
    global nome_disp_serial 
    global numero_disp_serial 
    global par_com_porta_serial 
    global status_porta_serial 

       
    foreach nome $args {
	if { [info exists ID_porta_serial($nome)] } {
	    catch { close $ID_porta_serial($nome) } ;#Fecha o identificador
	    unset ID_porta_serial($nome)  ;#Deleta os dados do objeto
	}
	if { [info exists nome_disp_serial($nome)] } {
	    unset nome_disp_serial($nome)   ;#Deleta os dados do objeto
	}
	if { [info exists numero_disp_serial($nome)] } {
	    unset numero_disp_serial($nome) ;#Deleta os dados do objeto
	}
	if { [info exists par_com_porta_serial($nome)] } {
	    unset par_com_porta_serial($nome) ;#Deleta os dados do objeto
	}
	if { [info exists status_porta_serial($nome)] } {
	    unset status_porta_serial($nome) ;#Deleta os dados do objeto
	}
	rename $nome {}       ;#Deleta o comando objeto

	puts "Removendo $nome"
    }
}


puts "pacote objLab carregado"


